# -*- encoding: utf-8 -*-

from django import forms
from django.forms.widgets import Select
from models import NOM_CHAMPS_ETAT, TEXTE_CHAMPS_ETAT, HELP_CHAMPS_ETAT, WorkflowCommentaire

def form_initialiser_etats(form, request, instance=None):
    """
    Modifier le champs 'etat' à travers le form, pour lui fournir la liste
    des états en fonction du contexte.
    """
    # ModelForm
    fields = getattr(form, 'fields', None)
    # AdminForm
    if not fields:
        fields = getattr(form, 'base_fields', None)
    if instance:
        choices = instance.get_etats_disponibles(request)
        fields[NOM_CHAMPS_ETAT].choices = choices
        fields[NOM_CHAMPS_ETAT].widget = Select(choices=choices)
    else:
        etat_initial = form.Meta.model.etat_initial
        libelle_etat_initial = form.Meta.model.etats[etat_initial]
        fields[NOM_CHAMPS_ETAT].initial = etat_initial
        choices = [(etat_initial, libelle_etat_initial), ]

        fields[NOM_CHAMPS_ETAT].choices = choices
        fields[NOM_CHAMPS_ETAT].widget = Select(choices=choices)


class WorkflowFormMixin(forms.ModelForm):
    """
    Mixin pour crééer un formulaire avec l'état dans le frontend.
    Pour faire un formulaire dans l'application, spécifier le modèle ainsi que les
    champs à afficher.
    """
    bouton_libelles = {}
    etat = forms.ChoiceField(label=TEXTE_CHAMPS_ETAT, help_text=HELP_CHAMPS_ETAT, required=False)
    commentaire = forms.CharField(widget=forms.Textarea, required=False)
    
    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop('request', None)
        if self.request is None:
            raise Exception("Workflow form doit est construit en connaissant le request")
        instance = kwargs.get('instance', None)
        super(WorkflowFormMixin, self).__init__(*args, **kwargs)
        form_initialiser_etats(self, request=self.request, instance=instance)

    def clean(self):
        """
        Substitution des valeurs de boutons au champs 'état'
        """
        etats = [k for k,v in self.fields[NOM_CHAMPS_ETAT].choices]
        for k, v in self.data.items():
            if k in etats:
                self.cleaned_data[NOM_CHAMPS_ETAT] = k
        return self.cleaned_data

    def save(self, *args, **kwargs):
       commentaire = WorkflowCommentaire()
       commentaire.content_object = self.instance
       commentaire.texte = self.data.get('commentaire', '')
       commentaire.etat_initial = self.instance._etat_courant
       commentaire.etat_final = self.instance.etat
       commentaire.owner = self.request.user
       commentaire.save()
       return super(forms.ModelForm, self).save(*args, **kwargs)

    def get_etats_disponibles(self, sans_etat_courant=True):
        return [(k, v) for k, v in self.fields[NOM_CHAMPS_ETAT].choices \
                if sans_etat_courant and self.instance.etat != k]

    def get_input_etats_as_buttons(self,):
        html = ""
        # on oblige la validation, le commentaire sans changement
        # d'étape n'est pas permis
        etats = self.get_etats_disponibles()
        if len(etats) == 0:
            return u"Aucune action n'est disponible pour vous à cette étape"
        for k, v in etats :
            if self.bouton_libelles.has_key(k):
                label = self.bouton_libelles[k]
            else:
                label = v
            html += u"<input type='submit' name='%s' value='%s' />" % (k, label)
        return html

        

